import numpy as np
import os
from os.path import join
from skimage import transform, io
from tqdm import tqdm
import torch
from segment_anything import sam_model_registry
from segment_anything.utils.transforms import ResizeLongestSide

# ============================ 参数配置区域 ============================
# 1. Mask文件夹路径[todo]
gt_path = ".../masks"
# 2. 保存npz文件路径[todo]
save_path = ".../npz"
# 3. 当前GPU设备号
os.environ['CUDA_VISIBLE_DEVICES'] = '0'
# 4. 模型类型
model_type = 'vit_h'
# 5. 初始权重文件路径[todo]
checkpoint = '.../sam_vit_h_4b8939.pth'
# 6. 原图image文件夹路径[todo]
img_path = ".../image"

# ============================ 模型和设备初始化 ============================
device = 'cuda' if torch.cuda.is_available() else 'cpu'
sam_model = sam_model_registry[model_type](checkpoint=checkpoint).to(device)
os.makedirs(save_path, exist_ok=True)

# 获取所有GT图像名称
names = sorted(os.listdir(gt_path))
imgs = []
gts = []
img_embeddings = []

# ============================ 数据处理和特征提取 ============================
for gt_name in tqdm(names, desc="Processing images"):
    # 修改图像文件名后缀（根据需求替换为实际格式）
    image_name = gt_name.split('.')[0] + ".bmp"
    
    # 读取GT
    gt_data = io.imread(join(gt_path, gt_name))
    if len(gt_data.shape) == 3:  # 如果是3D，取第1通道
        gt_data = gt_data[:, :, 0]
    assert len(gt_data.shape) == 2, 'GT must be 2D'

    # 处理GT数据（转为255，resize到256x256）
    gt_data = transform.resize(gt_data == 255, (256, 256), order=0, preserve_range=True, mode='constant')
    gt_data = np.uint8(gt_data)

    # 过滤GT面积过小的情况
    if np.sum(gt_data) > 100:
        assert np.max(gt_data) == 1 and np.unique(gt_data).shape[0] == 2, 'GT must contain binary values'

        # 读取原图
        image_data = io.imread(join(img_path, image_name))
        lower_bound, upper_bound = np.percentile(image_data, 0.5), np.percentile(image_data, 99.5)
        image_data_pre = np.clip(image_data, lower_bound, upper_bound)
        image_data_pre = (image_data_pre - np.min(image_data_pre)) / (np.max(image_data_pre) - np.min(image_data_pre)) * 255.0
        image_data_pre[image_data == 0] = 0
        image_data_pre = transform.resize(image_data_pre, (256, 256), order=3, preserve_range=True, mode='constant', anti_aliasing=True)
        image_data_pre = np.uint8(image_data_pre)

        imgs.append(image_data_pre)
        gts.append(gt_data)

        # SAM特征提取
        sam_transform = ResizeLongestSide(sam_model.image_encoder.img_size)
        resize_img = sam_transform.apply_image(image_data_pre)
        if len(resize_img.shape) == 2:
            resize_img = resize_img[:, :, np.newaxis]
        resize_img_tensor = torch.as_tensor(resize_img.transpose(2, 0, 1)).to(device)
        input_image = sam_model.preprocess(resize_img_tensor[None, :, :, :])
        
        with torch.no_grad():
            embedding = sam_model.image_encoder(input_image)
            img_embeddings.append(embedding.cpu().numpy()[0])

# ============================ 保存到NPZ ============================
if imgs and gts and img_embeddings:
    imgs = np.stack(imgs, axis=0)  # (n, 256, 256, 3)
    gts = np.stack(gts, axis=0)  # (n, 256, 256)
    img_embeddings = np.stack(img_embeddings, axis=0)  # (n, 256, 64, 64)
    for i in range(len(imgs)):
        save_file = join(save_path, f"data_{i}.npz")
        np.savez_compressed(save_file, imgs=imgs[i], gts=gts[i], img_embeddings=img_embeddings[i])
    print(f"NPZ files saved to {save_path}.")
else:
    print("No valid data found. Check your input images and masks.")
